class AR_Sprite : public AR_Bitmap
 {
  protected:
   
   bool need_collision_mask;
   int collision_mask_type,collision_mask_width,collision_mask_height;

   // Animate variables
   int current_frame,start_frame,end_frame;
   int animate_action;
   int animate_delay,animate_count;

   // Move variables
   float speed_x,speed_y,gravity_x,gravity_y;
   float friction_x,friction_y,acceleration_x,acceleration_y;
   float start_x,start_y,end_x,end_y;
   int move_action;

   // Rotate variables
   //int rotate_action;
   //TODO fare funzioni raotateTo, startRotation, stopRotation...

   // Scale variables
   int scale_action;
   float scale_speed,scale_end;

   // Scroll synchrony variables
   float *scroll_x, *scroll_y;

   int action; // Generic action (to use in classes that extends AR_Sprite)

   int rows,columns,width,height;     // Sprite frame size in bitmap file
   ALLEGRO_BITMAP **frames;           // Sprite frames
   ALLEGRO_BITMAP *collision_buffer;  // Collision buffer

   void reset()
    {
	 // Reset variables to default
	 AR_Bitmap::reset();
     start_x        = start_y        = 0;
     end_x          = end_y          = 0;
     gravity_x      = gravity_y      = 0;
     speed_x        = speed_y        = 0;
     friction_x     = friction_y     = 0;
     acceleration_x = acceleration_y = 0;
     animate_delay  = animate_count  = 0;
     life           = 100;
     scale_speed    = 0;
     scale_end      = 0;
     current_frame  = start_frame    = end_frame = 0;
     animate_action = AR::NONE;
     move_action    = AR::PHYSICS;
     scale_action   = AR::NONE;
     action         = AR::NONE;
	}

  public:

   AR_Sprite(string filename, AR_Display *display, int x, int y, AR_Sprite *p = NULL, int collision_mask_type = AR::NO_COLLISION_MASK) : AR_Bitmap(filename,display,p)
    {
	 reset();
     this->x = x;
     this->y = y;
     scroll_x = scroll_y = NULL;
     loadSprite(filename,p);

     // Init collison mask
     this->collision_mask_type = collision_mask_type;
     need_collision_mask       = false;
     collision_mask_width      = width;
     collision_mask_height     = height;
     if(collision_mask_type == AR::STATIC_COLLISION_MASK)
      {
       collision_buffer = al_create_bitmap(width,height);
       collision_buffer = frames[0];
      }
     if(collision_mask_type == AR::DYNAMIC_COLLISION_MASK)
      {
       collision_buffer = al_create_bitmap(max(width,height)*2,max(width,height)*2);
       need_collision_mask = true;
      }
    }

   ~AR_Sprite() { }

   void loadSprite(string filename, AR_Sprite *p)
    {
	 // Share or not share data with other AR_Sprite objects
     if(!p)
      {
       loadData(filename);
       loadFrames(filename);
      }
     else
      {
	   rows    = p->getRows();
	   columns = p->getColumns();
	   width   = p->getWidth();
       height  = p->getHeight();
       frames  = p->frames;
      }
    }

   void loadData(string filename)
    {
	 string   aux;
     ifstream fd;

     filename = AR_Util::cutExtension(filename);
     filename += ".dat";
     fd.open(filename.c_str());
     if(!fd.is_open()) { AR_Util::errorBox(display->getDisplay(),string("File '") + string(filename) + string("' not found!")); exit(1); }
     fd >> rows >> aux >> columns >> aux >> width >> aux >> height;
     fd.close();
    }

   void loadFrames(string filename)
    {
     string str;

     frames = (ALLEGRO_BITMAP **)malloc(sizeof(ALLEGRO_BITMAP *)*(rows*columns));
     for(int i=0;i<rows;i++)
      for(int j=0;j<columns;j++)
       {
        frames[j+columns*i] = al_create_bitmap(width,height);
        al_set_target_bitmap(frames[j+columns*i]);
        al_draw_bitmap_region(bitmap,width*j,height*i,width,height,0,0,0);
       }
     al_set_target_bitmap(al_get_backbuffer(display->getDisplay()));
     al_destroy_bitmap(bitmap); // Destroy bitmap loaded in class AR_Bitmap
    }

   void debug()
    {
	 float scroll_x, scroll_y;

	 if(!enable) return;

     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;

	 if(collision_mask_type == AR::NO_COLLISION) return;
     if(collision_mask_type != AR::NO_COLLISION_MASK) al_draw_bitmap(collision_buffer,x-scroll_x-(collision_mask_width>>1),y-scroll_y-(collision_mask_height>>1),0);
     al_draw_rectangle(x-scroll_x-(collision_mask_width>>1),y-scroll_y-(collision_mask_height>>1),x-scroll_x+(collision_mask_width>>1),y-scroll_y+(collision_mask_height>>1),al_map_rgb(255,0,0),1);
	}

   void run()
    {
	 if(!enable) return;

     // Animate
     if(animate_action==AR::ANIMATE)
      {
	   animate_count--;
	   if(animate_count <= 0) { animate_count = animate_delay; current_frame++; }
	   if(current_frame>end_frame) current_frame = start_frame;
	  }
     if(animate_action==AR::ANIMATE_TO)
      {
	   animate_count--;
	   if(animate_count <= 0) { animate_count = animate_delay; current_frame++; }
	   if(current_frame>end_frame) { current_frame = end_frame; animate_action = AR::NONE; }
	  }

     // Scale
     if(scale_action == AR::SCALE_TO)
	  {
       scale_x += scale_speed;
       scale_y += scale_speed;
       if(scale_speed > 0 && (scale_x > scale_end || scale_y > scale_end))
        {
         scale_action = AR::NONE;
         scale_x = scale_y = scale_end;
        }
       if(scale_speed < 0 && (scale_x < scale_end || scale_y < scale_end))
        {
         scale_action = AR::NONE;
         scale_x = scale_y = scale_end;
        }
	  }

     // Move
     if(move_action==AR::PHYSICS)
      {
       x+=speed_x;
       y-=speed_y;
       speed_x+=gravity_x;
       speed_y+=gravity_y;
       speed_x-=friction_x * speed_x;
       speed_y-=friction_y * speed_y;
       speed_x+=acceleration_x;
       speed_y+=acceleration_y;

       if(speed_x != 0) angle = AR_Util::degrees(atan2(-speed_y,speed_x));
      }
	 if(move_action == AR::MOVE_TO)
	  {
       bool gtx,gty,stop;

       x+=speed_x;
       y-=speed_y;
       stop = false;
       if(end_x >= start_x) gtx = true; else gtx = false;
       if(end_y >= start_y) gty = true; else gty = false;
       if(gtx  &&  gty) if(x >= end_x && y >= end_y) stop = true;
       if(gtx  && !gty) if(x >= end_x && y <= end_y) stop = true;
       if(!gtx &&  gty) if(x <= end_x && y >= end_y) stop = true;
       if(!gtx && !gty) if(x <= end_x && y <= end_y) stop = true;
       if(stop)
        {
		 x = end_x;
		 y = end_y;
         speed_x = speed_y = 0;
		 move_action == AR::PHYSICS;
		}
	  }
    }

   void draw()
    {
	 float scroll_x, scroll_y;
	 if(!enable) return;

     checkValues();
     scroll_x = scroll_y = 0;
     if(this->scroll_x != NULL) scroll_x = *this->scroll_x;
     if(this->scroll_y != NULL) scroll_y = *this->scroll_y;
	 al_draw_tinted_scaled_rotated_bitmap(frames[current_frame],
	                                      al_map_rgba(alpha * enable_tint_r,
	                                                  alpha * enable_tint_g,
	                                                  alpha * enable_tint_b,
	                                                  alpha),
	                                      center_x,center_y,
	                                      x-scroll_x,y-scroll_y,
	                                      scale_x,scale_y,
	                                      AR_Util::radians(angle),
	                                      flip_horizontal | flip_vertical);

     if(collision_mask_type == AR::DYNAMIC_COLLISION_MASK)
      {
       float oW,oH;
       float radians,radians90;

       radians   = AR_Util::radians(angle);
       radians90 = AR_Util::radians(90-angle);
       oW = al_get_bitmap_width (frames[current_frame]) * scale_x;
       oH = al_get_bitmap_height(frames[current_frame]) * scale_y;
       collision_mask_width  = (int)(abs(oW * cos(radians)) + abs(oH * cos(radians90)));
       collision_mask_height = (int)(abs(oW * sin(radians)) + abs(oH * sin(radians90)));

       if(need_collision_mask)
        {
	     al_set_target_bitmap(collision_buffer);
	     al_clear_to_color(al_map_rgba(0,0,0,0));
	     al_draw_tinted_scaled_rotated_bitmap(frames[current_frame],
	                                          al_map_rgba(0,0,0,255),
	                                          center_x,center_y,
	                                          collision_mask_width>>1,collision_mask_height>>1,
	                                          scale_x,scale_y,
	                                          radians,
	                                          flip_horizontal | flip_vertical);
         al_set_target_bitmap(al_get_backbuffer(display->getDisplay()));
         need_collision_mask = false;
	    }
      }
    }

   void moveTo(float x, float y, float speed)
    {
	 if(move_action==AR::PHYSICS)
	  {
	   float dx,dy,angle;

	   move_action = AR::MOVE_TO;
	   start_x = this->x; start_y = this->y;
	   end_x = x; end_y = y;
	   dx = end_x-start_x;
	   dy = end_y-start_y;
	   angle = atan2(dy,dx);
	   speed_x = speed*(cos(angle));
       speed_y = speed*(-sin(angle));
      }
	}

   void startAnimation(int start_frame, int end_frame, int delay = 0)
    {
	 if(animate_action==AR::NONE)
	  {
	   animate_action = AR::ANIMATE;
	   this->start_frame = start_frame;
	   this->end_frame   = end_frame;
	   current_frame     = start_frame;
	   animate_delay     = delay;
      }
    }

   void stopAnimation(int current_frame)
    {
     if(animate_action==AR::ANIMATE)
      {
	   animate_action      = AR::NONE;
	   this->current_frame = current_frame;
      }
    }

   void animateTo(int frame, int delay = 0)
    {
	 if(animate_action==AR::NONE)
	  {
	   animate_action = AR::ANIMATE_TO;
	   animate_delay  = delay;
	   end_frame      = frame;
      }
	}

//void startRotation(),stopRotatio(), rotateTo()...

   void scaleTo(float scale_end, float scale_speed)
    {
	 // scaleTo works fine only if scale_x and scale_y are the same
	 if(scale_action==AR::NONE)
	  {
	   if(scale_speed == 0)                             return; // No scale to do
       if(scale_x == scale_end || scale_y == scale_end) return; // No scale to do
       if(scale_x < scale_end  || scale_y < scale_end) this->scale_speed = scale_speed;
       if(scale_x > scale_end  || scale_y > scale_end) this->scale_speed = -scale_speed;
	   scale_action    = AR::SCALE_TO;
	   this->scale_end = scale_end;
      }
	}

   bool hitTest(float x, float y)
    {
	 // Test collision between AR_Sprite and (x,y) coordinates using collision_buffer
     ALLEGRO_COLOR c;
     unsigned char t,a;

     if(collision_mask_type == AR::NO_COLLISION) return false;
	 if(x >= this->x-(collision_mask_width>>1) && x <= this->x-(collision_mask_width>>1)+collision_mask_width)
	  if(y >= this->y-(collision_mask_height>>1) && y <= this->y-(collision_mask_height>>1)+collision_mask_height)
	   {
		if(collision_mask_type == AR::NO_COLLISION_MASK) return true;
        c = al_get_pixel(collision_buffer,(int)(x-((this->x)-(collision_mask_width>>1))),(int)(y-((this->y)-(collision_mask_height>>1))));
        al_unmap_rgba(c,&t,&t,&t,&a);
        if(a) return true;
        if(collision_mask_type == AR::DYNAMIC_COLLISION_MASK) need_collision_mask = true;
	   }
	 return false;
	}

   void setSpeedX(float speed_x)               { if(move_action==AR::PHYSICS) this->speed_x = speed_x;               }
   
   void setSpeedY(float speed_y)               { if(move_action==AR::PHYSICS) this->speed_y = speed_y;               }
   
   void setAccelerationX(float acceleration_x) { if(move_action==AR::PHYSICS) this->acceleration_x = acceleration_x; }

   void setAccelerationY(float acceleration_y) { if(move_action==AR::PHYSICS) this->acceleration_y = acceleration_y; }

   void setGravityX(float gravity_x)           { if(move_action==AR::PHYSICS) this->gravity_y = gravity_x;           }

   void setGravityY(float gravity_y)           { if(move_action==AR::PHYSICS) this->gravity_y = gravity_y;           }

   void setFrictionX(float friction_x)         { if(move_action==AR::PHYSICS) this->friction_x = friction_x;         }
   
   void setFrictionY(float friction_y)         { if(move_action==AR::PHYSICS) this->friction_y = friction_y;         }

   void setCurrentFrame(int frame)             { if(animate_action==AR::NONE) current_frame = frame;                 }

   void setScrollXP(float *scroll_x) { this->scroll_x = scroll_x; }
   
   void setScrollYP(float *scroll_y) { this->scroll_y = scroll_y; }

   float getSpeedX()        { return speed_x;        }
   
   float getSpeedY()        { return speed_y;        }

   float getAccelerationX() { return acceleration_x; }
   
   float getAccelerationY() { return acceleration_y; }

   float getGravityX()      { return gravity_x;      }
   
   float getGravityY()      { return gravity_y;      }

   float getFrictionX()     { return friction_x;     }
   
   float getFrictionY()     { return friction_y;     }
   
   int getRows()         { return rows;          }

   int getColumns()      { return columns;       }

   int getWidth()        { return width;         }

   int getHeight()       { return height;        }

   int getCurrentFrame() { return current_frame; }
   
   int getAction()       { return action;        }
   
   int getCollisionMaskType() { return collision_mask_type;  }

 };
